#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 10 17:02:36 2017

@author: steven
"""

import volmdlr as vm
import volmdlr.primitives2D as primitives2D
import time
p1=vm.Point2D((0,0))
p2=vm.Point2D((1,0))
p3=vm.Point2D((2,1))
p4=vm.Point2D((1,0.5))
p5=vm.Point2D((-1,0.1))

p=vm.Polygon2D([p1,p2,p3,p4,p5])

#print(c.Area())
#print(c.SecondMomentArea(p1))

import numpy as npy

points_inside=[]
points_outside=[]

for i in range(100):
    pt=vm.Point2D(2*npy.random.random(2)-0.3)
#    print(p.PointDistance(pt))
    if p.PointBelongs(pt):
        points_inside.append(pt)
    else:
        points_outside.append(pt)
 
    
p.MPLPlot()
#points_inside.MPLPlot()
c1=vm.CompositePrimitive2D([p,*points_inside])
c1.MPLPlot()

c2=vm.CompositePrimitive2D([p,*points_outside])
c2.MPLPlot()

cog_p=p.CenterOfMass()
cog_p.MPLPlot('ro')



# Speed test
t = time.time()
n = 100000
for i in range(n):
    pt=vm.Point2D(2*npy.random.random(2)-0.3)
#    print(p.PointDistance(pt))
    p.PointBelongs(pt)
t= time.time() - t 
print('time spent: {}s, {}s/eval'.format(t, t/n))   


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun 21 16:38:31 2017

@author: steven
"""

import volmdlr as vm
import math

r1=1.78*0.5
r2=r1+0.3

theta1=12*2*math.pi/360
theta2=33*2*math.pi/360


pm1=vm.Point2D((0,-r1))
pm2=vm.Point2D((0,-r2))
pc=vm.Point2D((0,0))
p1=pm1.Rotation(pc,-theta2)
p2=pm1.Rotation(pc,-theta1)
p3=pm1.Rotation(pc,theta1)
p4=pm1.Rotation(pc,theta2)

p8=pm2.Rotation(pc,-theta2)
p7=pm2.Rotation(pc,-theta1)
p6=pm2.Rotation(pc,theta1)
p5=pm2.Rotation(pc,theta2)

border=vm.Polygon2D([p1,p2,p3,p4,p5,p6,p7,p8],'border')

ptest=vm.Point2D((-0.08366,-0.91306))

projs=[]
for line in border.Lines():
    projs.append(line.PointProjection(ptest))



#print('test: ',border.PointDistance(ptest)+(ptest.vector[1]-p2.vector[1]))

ctest=vm.CompositePrimitive2D([border,ptest]+projs)
ctest.MPLPlot()
cog_p=border.CenterOfMass()
cog_p.MPLPlot('ro')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun  9 16:26:30 2017

@author: steven
"""
#from libcpp cimport bool

def PolygonPointBelongs(point,points):
    
    cdef int n = len(points)
    cdef bint inside = False
    cdef float x, y, p1x, p1y, p2x, p2y, xints
    x,y=point
    p1x,p1y = points[0]
    
    for i in range(n+1):
        p2x, p2y = points[i % n]
        if y > min(p1y,p2y):
            if y <= max(p1y,p2y):
                if x <= max(p1x,p2x):
                    if p1y != p2y:
                        xints = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
                    if p1x == p2x or x <= xints:
                        inside = not inside
        p1x, p1y = p2x, p2y
    
    return inside


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar  5 22:01:35 2017

@author: steven


"""

#import numpy as npy
import volmdlr as vm
import volmdlr.primitives3D as primitives3D
import volmdlr.primitives2D as primitives2D
#import math

p1=vm.Point2D((0,0))
p2=vm.Point2D((0.1,0.))
p3=vm.Point2D((0.1,0.2))
p4=vm.Point2D((0,0.1))
p5=vm.Point2D((0.01,0.05))

#p6=vm.Point2D((0.1,0.3))

l1=primitives2D.RoundedLines2D([p1,p2,p3,p4],{2:0.01},False)
l2=vm.Arc2D(p4,p5,p1)
c1=vm.Contour2D([l1,l2])
c1.MPLPlot()

po=vm.Point3D((0,0,0))
xp=vm.Vector3D((1,0,0))
yp=vm.Vector3D((0,1,0))


profile=primitives3D.ExtrudedProfile(po,xp,yp,[c1],(0,0.1,0.2))

model=vm.VolumeModel([profile])

#profile.MPLPlot((0,0,0),(1,0,0),(0,1,0))

#model.MPLPlot()

model.FreeCADExport('python','extrusion2','/usr/lib/freecad/lib/')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 14 15:32:37 2018

@author: Steven Masfaraud masfaraud@dessia.tech
"""
import volmdlr as vm
import volmdlr.primitives2D as primitives2D
import numpy as npy

#for i in range(20):
triangle_points=[vm.Point2D(npy.random.random(2)) for i in range(3)]
triangle=vm.Polygon2D(triangle_points)


cog_triangle=triangle.CenterOfMass()
triangle.MPLPlot()
cog_triangle.MPLPlot('ro')

print(triangle.Area())

p0=vm.Point2D((-0.5,0))
p1=vm.Point2D((0,0.5))
p2=vm.Point2D((0.5,0))

a=vm.Arc2D(p0,p1,p2)
l=vm.Line2D(p2,p0)

c=vm.Contour2D([a,l])

c.MPLPlot()
print(c.Area())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 28 14:07:37 2017

@author: steven
"""

import math
import numpy as npy
import matplotlib.pyplot as plt
from matplotlib.patches import Arc
from mpl_toolkits.mplot3d import Axes3D
#import vmcy
from .vmcy import PolygonPointBelongs

from scipy.linalg import norm,solve,LinAlgError

import volmdlr.geometry as geometry

from jinja2 import Environment, PackageLoader, select_autoescape

import webbrowser
import os

import tempfile
import subprocess


class Vector2D:
    def __init__(self,vector):
#        print(vector,npy.array(vector))
#        self.vector=npy.array([vector[0],vector[1]])
        self.vector=npy.zeros(2)
        self.vector[0]=vector[0]        
        self.vector[1]=vector[1]
        
    def To3D(self,plane_origin,x1,x2):
        x,y=self.vector
        return Point3D(plane_origin.vector+x1.vector*x+x2.vector*y)
    
    def __add__(self,point2d):
        return Vector2D(self.vector+point2d.vector)
    
    def __radd__(self,point2d):
        return self+point2d

    def __sub__(self,point2d):
        return Vector2D(self.vector-point2d.vector)
    
    def __rsub__(self,point2d):
        return self-point2d    
    
    def __mul__(self,value):
        return Vector2D(self.vector*value)
    
    def __rmul__(self,value):
        return self*value

    def __truediv__(self,value):
        return Vector2D(self.vector/value)
    
    def __rtruediv__(self,value):
        return self/value
    
    def Rotation(self,center,angle,copy=True):
        vector2=npy.dot(npy.array([[math.cos(angle),-math.sin(angle)],[math.sin(angle),math.cos(angle)]]),(self.vector-center.vector))+center.vector
        if copy:
            return Point2D(vector2)
        else:
            self.vector=vector2

    def Translation(self,offset,copy=True):
        vector2=self.vector+offset
        if copy:
            return Point2D(vector2)
        else:
            self.vector=vector2
        

class Point2D(Vector2D):
    def __init__(self,vector,name=''):
        Vector2D.__init__(self,vector)
        self.name=name
        
    def __add__(self,point2d):
        return Point2D(self.vector+point2d.vector)
    
    def __sub__(self,point2d):
        return Point2D(self.vector-point2d.vector)
        
    def __mul__(self,value):
        return Point2D(self.vector*value)
    
    def __truediv__(self,value):
        return Point2D(self.vector/value)
    

    def MPLPlot(self,style='ob'):
        x1=self.vector
        plt.plot([x1[0]],[x1[1]],style)        
        return []
    
    def PointDistance(self,point2):
        return norm(self.vector-point2.vector)

    #def Distance(self,point2):
    #    return norm(self.vector-point2.vector)

    @classmethod
    def LinesIntersection(cls,line1,line2,curvilinear_abscissa=False):
        p11=line1.points[0].vector
        p12=line1.points[1].vector
        p21=line2.points[0].vector
        p22=line2.points[1].vector
        A=npy.array([[p12[0]-p11[0],p21[0]-p22[0]],[p12[1]-p11[1],p21[1]-p22[1]]])
        x=npy.array([p21[0]-p11[0],p21[1]-p11[1]])
        try:
            t=solve(A,x)
            if not curvilinear_abscissa:
                return cls(p11+t[0]*(p12-p11))
            else:
                return (cls(p11+t[0]*(p12-p11)),t[0],t[1])
        except LinAlgError:
            return None
        
    @classmethod
    def MiddlePoint(cls,point1,point2):
        p1=point1.vector
        p2=point2.vector
        return cls((p1+p2)*0.5)

    @classmethod
    def LineProjection(cls,point,line):
        p1,p2=line.points
        d=(p2-p1)/p2.PointDistance(p1)
        n=d.Rotation(Point2D((0,0)),math.pi/2).vector
        pp1=point.vector-p1.vector
        p=pp1-npy.dot(pp1,n)*n+p1.vector
        return Point2D(p)
        
class Primitive2D:
    def __init__(self,name=''):
        self.name=name
        
class CompositePrimitive2D(Primitive2D):
    """
    A collection of simple primitives
    """
    def __init__(self,primitives,name=''):
        Primitive2D.__init__(self,name)        
        self.primitives=primitives
        
    def Rotation(self,center,angle,copy=False):
        if copy:
            return self.__class__([p.Rotation(center,angle,copy=True) for p in self.primitives])
        else:
            for p in self.primitives:
                p.Rotation(center,angle,copy=False)
            
    def Translation(self,offset,copy=False):
        if copy:
            return self.__class__([p.Translation(offset,copy=True) for p in self.primitives])
        else:
            for p in self.primitives:
                p.Translation(offset,copy=False)
    
        
    def MPLPlot(self):
#        lines,arcs=self.Export2D(point,x1,x2)
        fig, ax = plt.subplots()
        ax.set_aspect('equal')
        ps=[]
        for element in self.primitives:
            ps.extend(element.MPLPlot())
#        p = PatchCollection(ps)
#        print(arcs)
        for p in ps:
            ax.add_patch(p)
        
        ax.margins(0.1)
        plt.show() 
        
class Contour2D(CompositePrimitive2D):
    """
    A collection of primitives intended to be closed
    """
    def __init__(self,primitives,name=''):
        
        primitives2=[]
        for primitive in primitives:
            try:
                primitives2.extend(primitive.primitives)
            except AttributeError:
                primitives2.append(primitive)

        CompositePrimitive2D.__init__(self,primitives2,name)
        
    def To3D(self,plane_origin,x,y):
        return [primitive.To3D(plane_origin,x,y) for primitive in self.primitives]
    
    def Area(self):
        if len(self.primitives)==1:
            return self.primitives[0].Area()
        arcs=[]
        points_polygon=[]
        for primitive in self.primitives:
#            print(primitive.__class__.__name__)
            if primitive.__class__.__name__=='Line2D':
                points_polygon.extend(primitive.points)
            elif primitive.__class__.__name__=='Arc2D':
                points_polygon.append(primitive.center)
                arcs.append(primitive)
        polygon=Polygon2D(points_polygon)
        A=polygon.Area()
#        print('A',A)
#        print('arcs',arcs)
        for arc in arcs:
#            print(arc,arc.Area())
            if polygon.PointBelongs(arc.middle):
                A-=arc.Area()
            else:
                A+=arc.Area()
                
        return A
    
    def CenterOfMass(self):
        if len(self.primitives)==1:
            return self.primitives[0].CenterOfMass()

        arcs=[]
        points_polygon=[]
        for primitive in self.primitives:
            if primitive.__class__.__name__=='Line2D':
                points_polygon.extend(primitive.points)
            elif primitive.__class__.__name__=='Arc2D':
                points_polygon.append(primitive.center)
                arcs.append(primitive)
        polygon=Polygon2D(points_polygon)
        
        area=polygon.Area()
        c=area*polygon.CenterOfMass()
        
        for arc in arcs:
            arc_area=arc.Area()
            if polygon.PointBelongs(arc.middle):
                c-=arc_area*arc.CenterOfMass()
                area-=arc_area
            else:
                c+=arc_area*arc.CenterOfMass()
                area+=arc_area
        return c/area



    def SecondMomentArea(self,point):
        if len(self.primitives)==1:
            return self.primitives[0].SecondMomentArea(point)

        arcs=[]
        points_polygon=[]
        for primitive in self.primitives:
            if primitive.__class__.__name__=='Line2D':
                points_polygon.extend(primitive.points)
            elif primitive.__class__.__name__=='Arc2D':
                points_polygon.append(primitive.center)
                arcs.append(primitive)
        polygon=Polygon2D(points_polygon)
        A=polygon.SecondMomentArea(point)
        for arc in arcs:
            if polygon.PointBelongs(arc.middle):
                A-=arc.SecondMomentArea(point)
            else:
                A+=arc.SecondMomentArea(point)
        return A

class Mesh2D:
    def __init__(self,contours,points_densities,default_density):
        self.contours=contours
        self.points_densities=points_densities
        self.default_density=default_density
        
    def GeoScript(self,filepath=''):
        s=''
        ipt=1# point index
        ipr=1# primitive index 
        points_index={}
        #assigning an index to point    
        for contour in self.contours:
            for primitive in contour.primitives:
                for point in primitive.geo_points:
                    try:
                        points_index[point]
                    except KeyError:
                        points_index[point]=ipt
                        try:
                            d=self.points_densities[point]
                        except KeyError:
                            d=self.default_density
                        s+='Point({})={{{},{},0.,{}}};\n'.format(ipt,*point.vector,d)
                        ipt+=1
        contours_indices=[]
        for contour in self.contours:
            contour_iprs=[]
            for primitive in contour.primitives:
                spr,ipr2=primitive.GeoScript(ipr,[points_index[p] for p in primitive.geo_points])
                s+=spr
                contour_iprs.extend(range(ipr,ipr2))
                ipr=ipr2
            s+='Line Loop({}) = {{{}}};\n'.format(ipr,str(contour_iprs)[1:-1])
            contours_indices.append(ipr)
            ipr+=1
        s+='Plane Surface({}) = {{{}}};\n'.format(ipr,str(contours_indices)[1:-1])
        # Saving to file if required
        if filepath!='':
            with open(filepath,'w') as file:
                file.write(s)
        return s

class Line2D(Primitive2D):
    def __init__(self,point1,point2,name=''):
        Primitive2D.__init__(self,name)        
        self.points=[point1,point2]
        
    def _get_geo_points(self):
        return self.points

    geo_points=property(_get_geo_points)      
    
    def PointProjection(self,point,curvilinear_abscissa=False):
        p1,p2=self.points
        v=p1.vector
        w=p2.vector
        p=point.vector
        t = npy.dot(p - v, w - v) / norm(w-v)**2
#        print('t', t)
        projection = Point2D(v + t * (w - v))# Projection falls on the segment
        if curvilinear_abscissa:
            return projection,t
        return projection
    
    def PointSegmentDistance(self,point):
        """
        Computes the distance of a point to segment of line 
        """
        p1,p2=self.points
        v=p1.vector
        w=p2.vector
        p=point.vector
#        print('point vector',point.vector)
        nwv2=(w[1]-v[1])**2+(w[0]-v[0])**2# replace norm(w-v)**2
        t = max(0, min(1, npy.dot(p - v, w - v) / nwv2))
#        print('t', t)
        projection = v + t * (w - v)# Projection falls on the segment
#        print(p,projection)
#        return norm(p-projection);
        return ((p[1]-projection[1])**2+(p[0]-projection[0])**2)**0.5
    
    def PointDistance(self,point):
        """
        Computes the distance of a point to line 
        """
        p1,p2=self.points
        v=p1.vector
        w=p2.vector
        p=point.vector
#        print('point vector',point.vector)
        t = npy.dot(p - v, w - v) / norm(w-v)**2
#        print('t', t)
        projection = v + t * (w - v)# Projection falls on the segment
#        print(p,projection)
#        return norm(p-projection);
        return ((p[1]-projection[1])**2+(p[0]-projection[0])**2)**0.5


    def MPLPlot(self):
        p1,p2=self.points
        x1=p1.vector
        x2=p2.vector
        plt.plot([x1[0],x2[0]],[x1[1],x2[1]],'black')        
        return []
    
    def To3D(self,plane_origin,x1,x2):
        p3D=[p.To3D(plane_origin,x1,x2) for p in self.points]
        return Line3D(*p3D,self.name)
    
    def Rotation(self,center,angle,copy=False):
        if copy:
            return Line2D(*[p.Rotation(center,angle,copy=True) for p in self.points])
        else:
            for p in self.points:
                p.Rotation(center,angle,copy=False)
            
    def Translation(self,offset,copy=False):
        if copy:
            return Line2D(*[p.Translation(offset,copy=True) for p in self.points])
        else:
            for p in self.points:
                p.Translation(offset,copy=False)
                
    def GeoScript(self,primitive_index,points_indices):
        s='Line({}) = {{{}, {}}};\n'.format(primitive_index,*points_indices)
        return s,primitive_index+1
                
                
class Arc2D(Primitive2D):
    def __init__(self,start,middle,end,name=''):        
        Primitive2D.__init__(self,name)        
        self.middle=middle
        self.start=start
        self.end=end
        xm,ym=middle.vector
        xe,ye=end.vector
        xs,ys=start.vector
        A=npy.array([[2*(xs-xm),2*(ys-ym)],
                   [2*(xs-xe),2*(ys-ye)]])
        b=-npy.array([xm**2+ym**2-xs**2-ys**2,xe**2+ye**2-xs**2-ys**2])
        self.center=Point2D(solve(A,b))
        r1=self.start.vector-self.center.vector
        r2=self.end.vector-self.center.vector
        rc=self.middle.vector-self.center.vector
        
        self.radius=norm(r1)
        angle1=npy.arctan2(r1[1],r1[0])
        angle2=npy.arctan2(r2[1],r2[0])
#        angle_min=min(angle1,angle2)
#        angle_max=max(angle1,angle2)
        anglem=npy.arctan2(rc[1],rc[0])
        order=[y for x,y in sorted(zip([angle1,anglem,angle2],[0,1,2]))]
        order=order*2
        i=order.index(0)
        if order[i+1]==1:
            self.angle1=angle1
            self.angle2=angle2
        else:
            self.angle1=angle2
            self.angle2=angle1
            
        
    def _get_geo_points(self):
        return [self.start,self.center,self.end]

    geo_points=property(_get_geo_points)        
    
    def GeoScript(self,primitive_index,points_indices):
        s='Circle({}) = {{{}, {}, {}}};\n'.format(primitive_index,*points_indices)
        return s,primitive_index+1
            
    def Area(self):
        if self.angle2<self.angle1:
            angle=self.angle2+2*math.pi-self.angle1
        else:
            angle=self.angle2-self.angle1
        return self.radius**2*angle/2
            
    def CenterOfMass(self):
        u=self.middle.vector-self.center.vector
        u/=norm(u)
        alpha=abs(self.angle1-self.angle2)
        return Point2D(self.center.vector+4/(3*alpha)*self.radius*math.sin(alpha*0.5)*u)
        
    def MPLPlot(self):
        pc=self.center.vector
        return [Arc(pc,2*self.radius,2*self.radius,angle=0,theta1=self.angle1*0.5/math.pi*360,theta2=self.angle2*0.5/math.pi*360,color='black')]

    def To3D(self,plane_origin,x,y):
        pc=self.center.To3D(plane_origin,x,y)
        pe=Point2D(self.center.vector+self.radius*npy.array((math.cos(self.angle1),math.sin(self.angle1))))
        pe3=pe.To3D(plane_origin,x,y)
        ps=Point2D(self.center.vector+self.radius*npy.array((math.cos(self.angle2),math.sin(self.angle2))))
        ps3=ps.To3D(plane_origin,x,y)
        return Arc3D(pe3,pc,ps3,self.name)
    
    def Rotation(self,center,angle,copy=False):
        if copy:
            return Arc2D(*[p.Rotation(center,angle,copy=True) for p in [self.start,self.middle,self.end]])
        else:
            self.__init__(*[p.Rotation(center,angle,copy=True) for p in [self.start,self.middle,self.end]])
            
    def Translation(self,offset,copy=False):
        if copy:
            return Arc2D(*[p.Translation(offset,copy=True) for p in [self.start,self.middle,self.end]])
        else:
            self.__init__(*[p.Translation(offset,copy=True) for p in [self.start,self.middle,self.end]])

    def SecondMomentArea(self,point):
        """ 
        Second moment area of part of disk
        """
        if self.angle2<self.angle1:
            angle2=self.angle2+2*math.pi
            
        else:
            angle2=self.angle2
        angle1=self.angle1

        Ix=self.radius**4/8*(angle2-angle1+0.5*(math.sin(2*angle1)-math.sin(2*angle2)))
        Iy=self.radius**4/8*(angle2-angle1+0.5*(math.sin(2*angle2)-math.sin(2*angle1)))
        Ixy=self.radius**4/8*(math.cos(angle1)**2-math.cos(angle2)**2)
        Ic=npy.array([[Ix,Ixy],[Ixy,Iy]])
        return geometry.Huygens2D(Ic,self.Area(),self.center,point)



class Circle2D(Primitive2D):
    def __init__(self,center,radius,name=''):        
        Primitive2D.__init__(self,name)        
        self.center=center
        self.radius=radius
        self.utd_geo_points=False
        
    def _get_geo_points(self):
        if not self.utd_geo_points:
            self._geo_start=self.center+self.radius*Point2D((1,0))
            self.utd_geo_points=True
        return [self._geo_start,self.center,self._geo_start]

    geo_points=property(_get_geo_points)        
            
    def GeoScript(self,primitive_index,points_indices):
        s='Circle({}) = {{{}, {}, {}}};\n'.format(primitive_index,*points_indices)
        return s,primitive_index+1
    
    def MPLPlot(self):
        pc=self.center.vector
        return [Arc(pc,2*self.radius,2*self.radius,angle=0,theta1=0,theta2=360,color='black')]

    def To3D(self,plane_origin,x,y):
        normal=Vector3D(npy.cross(x.vector,y.vector))
        pc=self.center.To3D(plane_origin,x,y)
        return Circle3D(pc,self.radius,normal,self.name)

    def Rotation(self,center,angle,copy=False):
        if copy:
            return Circle2D(self.center.vector.Rotation(center,angle,copy=True),self.radius)
        else:
            self.center.Rotation(center,angle,copy=False) 
            self.utd_geo_points=False
            
    def Translation(self,offset,copy=False):
        if copy:
            return Circle2D(self.center.vector.Translation(offset,copy=True),self.radius)
        else:
            self.center.Translation(offset,copy=False)
            self.utd_geo_points=False

    def Area(self):
        return math.pi*self.radius**2

    def SecondMomentArea(self,point):
        """ 
        Second moment area of part of disk
        """
        I=math.pi*self.radius**4/4
        Ic=npy.array([[I,0],[0,I]])
        return geometry.Huygens2D(Ic,self.Area(),self.center,point)
    
    def CenterOfMass(self):
        return self.center

class Polygon2D(CompositePrimitive2D):
    def __init__(self,points,name=''):     
        self.points=points
        primitives=[]
        for p1,p2 in zip(points,points[1:]+[points[0]]):
            primitives.append(Line2D(p1,p2))
            
            
        CompositePrimitive2D.__init__(self,primitives,name)
        
        
    def Area(self):
        
        x=[point.vector[0]for point in self.points]
        y=[point.vector[1]for point in self.points]

        return 0.5*npy.abs(npy.dot(x,npy.roll(y,1))-npy.dot(y,npy.roll(x,1)))
    
    def CenterOfMass(self):
        
        x=[point.vector[0] for point in self.points]
        y=[point.vector[1] for point in self.points]

        
        xi_xi1=x+npy.roll(x,-1)
        yi_yi1=y+npy.roll(y,-1)
        xi_yi1=npy.multiply(x,npy.roll(y,-1))
        xi1_yi=npy.multiply(npy.roll(x,-1),y)
        
        a=0.5*npy.sum(xi_yi1-xi1_yi)# signed area!
#        a=self.Area()
        
        cx=npy.sum(npy.multiply(xi_xi1,(xi_yi1-xi1_yi)))/6./a  
        cy=npy.sum(npy.multiply(yi_yi1,(xi_yi1-xi1_yi)))/6./a  

        return Point2D((cx,cy)) 
    
    def PointBelongs(self,point):
        """
        Ray casting algorithm copied from internet...
        """
        return PolygonPointBelongs(point.vector,[p.vector for p in self.points])

    def SecondMomentArea(self,point):
        Ix,Iy,Ixy=0,0,0
        for pi,pj in zip(self.points,self.points[1:]+[self.points[0]]):
            xi,yi=pi.vector-point.vector
            xj,yj=pj.vector-point.vector
            Ix+=(yi**2+yi*yj+yj**2)*(xi*yj-xj*yi)
            Iy+=(xi**2+xi*xj+xj**2)*(xi*yj-xj*yi)
            Ixy+=(xi*yj+2*xi*yi+2*xj*yj+xj*yi)*(xi*yj-xj*yi)
        if Ix<0:
            Ix=-Ix
            Iy=-Iy
            Ixy=-Ixy
        return npy.array([[Ix/12.,Ixy/24.],[Ixy/24.,Iy/12.]])

    def Lines(self):
        lines=[]
        for p1,p2 in zip(self.points,self.points[1:]+[self.points[0]]):
            lines.append(Line2D(p1,p2))
        return lines
    
    def MPLPlot(self):
        x=[]
        y=[]
        for p in self.points+[self.points[0]]:
            x.append(p.vector[0])
            y.append(p.vector[1])
        plt.plot(x,y,label=self.name)
            
#        for line in self.Lines():
#            line.MPLPlot()
        return []

    def PointDistance(self,point):
        lines=self.Lines()
        d_min=lines[0].PointSegmentDistance(point)
#        print('d: ',d_min,0)
        for line in lines[1:]:
            d=line.PointSegmentDistance(point)
#            print('d: ',d)
            if d<d_min:
                d_min=d
        return d_min
                

class Primitive3D:
    def __init__(self,name=''):
        self.name=name        

        
class Vector3D:
    def __init__(self,vector):
#        self.vector=npy.array(vector)
        self.vector=npy.zeros(3)
        self.vector[0]=vector[0]        
        self.vector[1]=vector[1]
        self.vector[2]=vector[2]
    
    def __add__(self,point2d):
        return self.__class__(self.vector+point2d.vector)
    
    def __radd__(self,point2d):
        return self+point2d

    def __sub__(self,point2d):
        return Vector3D(self.vector-point2d.vector)
    
    def __rsub__(self,point2d):
        return self-point2d    
    
    def __mul__(self,value):
        return Vector3D(self.vector*value)
    
    def __rmul__(self,value):
        return self*value

    def __div__(self,value):
        return Vector3D(self.vector/value)
    
    def __rdiv__(self,value):
        return self/value
    
    def Norm(self):
        x,y,z = self.vector
        return (x**2+y**2+z**2)**0.5
        
    def Rotation(self,center,axis,angle,copy=True):
        u=axis.vector
        ux=npy.array([[0,-u[2],u[1]],[u[2],0,-u[0]],[-u[1],u[0],0]])
        R=math.cos(angle)*npy.eye(3)+math.sin(angle)*ux+(1-math.cos(angle))*npy.tensordot(u,u,axes=0)
        vector2=npy.dot(R,(self.vector-center.vector))+center.vector
        if copy:
            return Point3D(vector2)
        else:
            self.vector=vector2

    def Translation(self,offset,copy=True):
        vector2=self.vector+offset
        if copy:
            return Point3D(vector2)
        else:
            self.vector=vector2    
            
    def RandomUnitNormalVector(self):
        """
        Returns a random normal vector
        """
        v=npy.random.random(3)
        
        v=Vector3D(v-npy.dot(v,self.vector)*self.vector/(self.Norm()**2))
        v.vector=v.vector/v.Norm()
        return v

        
class Point3D(Vector3D):
    def __init__(self,vector,name=''):
        Vector3D.__init__(self,vector)
        self.name=name
        
    def MPLPlot(self,ax):
        ax.scatter(*self.vector)
        
    def PlaneProjection(self,plane_origin,x,y):
        z=npy.cross(x.vector,y.vector)
        z/=norm(z)
        return Point3D(self.vector-npy.dot(self.vector-plane_origin.vector,z)*z)
        
    def To2D(self,plane_origin,x,y):
        if npy.dot(x.vector,y.vector)!=0:
            raise NotImplemented
        x2d=npy.dot(self.vector,x.vector)-npy.dot(plane_origin.vector,x.vector)
        y2d=npy.dot(self.vector,y.vector)-npy.dot(plane_origin.vector,y.vector)
        return Point2D((x2d,y2d))
    
    def PointDistance(self,point2):
        return norm(self.vector-point2.vector)
        
class Line3D(Primitive3D):
    def __init__(self,point1,point2,name=''):
        Primitive3D.__init__(self,name)        
        self.points=[point1,point2]
        
    def MPLPlot(self,ax):
        x=[p.vector[0] for p in self.points]
        y=[p.vector[1] for p in self.points]
        z=[p.vector[2] for p in self.points]
        ax.plot(x,y,z)
        
        
    def FreeCADExport(self,name,ndigits=3):
        x1,y1,z1=npy.round(1000*self.points[0].vector,ndigits)
        x2,y2,z2=npy.round(1000*self.points[1].vector,ndigits)
        return '{}=Part.LineSegment(fc.Vector({},{},{}),fc.Vector({},{},{}))\n'.format(name,x1,y1,z1,x2,y2,z2)


class Circle3D(Primitive3D):
    def __init__(self,center,radius,normal,name):        
        Primitive2D.__init__(self,name)        
        self.center=center
        self.radius=radius
        self.normal=normal

    def FreeCADExport(self,name,ndigits=3):
        xc,yc,zc=npy.round(1000*self.center.vector,ndigits)
        xn,yn,zn=npy.round(self.normal.vector,ndigits)
        return '{}=Part.Circle(fc.Vector({},{},{}),fc.Vector({},{},{}),{})\n'.format(name,xc,yc,zc,xn,yn,zn,1000*self.radius)
        

class Arc3D(Primitive3D):
    def __init__(self,start,center,end,name):        
        Primitive2D.__init__(self,name)        
        self.center=center
        self.start=start
        self.end=end
        r=norm(start.vector-center.vector)
        n=(start.vector-2*center.vector+end.vector)
        n=n/norm(n)
        self.middle=Point3D(r*n+center.vector)

    def MPLPlot(self,ax):
        ax.scatter(*self.center.vector,c='b')
        ax.scatter(*self.start.vector,c='r')
        ax.scatter(*self.end.vector,c='r')
        ax.scatter(*self.middle.vector,c='g')
        
        
    
    def FreeCADExport(self,name,ndigits=3):
        xs,ys,zs=npy.round(1000*self.start.vector,ndigits)
        xm,ym,zm=npy.round(1000*self.middle.vector,ndigits)
        xe,ye,ze=npy.round(1000*self.end.vector,ndigits)
        return '{}=Part.Arc(fc.Vector({},{},{}),fc.Vector({},{},{}),fc.Vector({},{},{}))\n'.format(name,xs,ys,zs,xm,ym,zm,xe,ye,ze)

        

class VolumeModel:
    def __init__(self,primitives,name=''):
        self.primitives=primitives
        self.name=name
        
    def Volume(self):
        volume=0
        for primitive in self.primitives:
            volume+=primitive.Volume()
        return volume
    
    def MPLPlot(self):
        """
        Matplotlib plot of model.
        To use for debug.
        """
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d',adjustable='box')
        ax.set_aspect('equal')
        for primitive in self.primitives:
            primitive.MPLPlot(ax)
#        ax.set_aspect('equal')
    
    def FreeCADScript(self,fcstd_filepath,path_lib_freecad='',py_filepath='',export_types=['fcstd']):
        """
        Generate python a FreeCAD definition of model 
        :param fcstd_filename: a filename without extension to give the name at the fcstd part written in python code
        :param py_filepath: python file to be exported. If none give, only return in string
        :type fcstd_filename:str
        :type py_filepath:str
        """
        fcstd_filepath=os.path.abspath(fcstd_filepath)
        s=''
        if path_lib_freecad!='':
            s+="import sys\nsys.path.append('"+path_lib_freecad+"')\n"

        s+="import math\nimport FreeCAD as fc\nimport Part\n\ndoc=fc.newDocument('doc')\n\n"
        
        for ip,primitive in enumerate(self.primitives):
            sp=primitive.FreeCADExport(ip)
            if sp!='':
                s+=(sp+'\n')
                s+='shapeobj = doc.addObject("Part::Feature","p{} {}")\n'.format(ip,primitive.name)
                s+="shapeobj.Shape = primitive{}\n".format(ip)
        s+='doc.recompute()\n'
        if 'fcstd' in export_types:
            s+="doc.saveAs('"+fcstd_filepath+".fcstd')\n\n"
        if 'stl' in export_types:
#            p_list=str(['primitive{}'.format(i) for i in range(len(self.primitives))]).replace("'","")
            s+="import Mesh\nMesh.export(doc.Objects,'{}.stl')\n".format(fcstd_filepath)
                

        if py_filepath!='':
            with open(py_filepath,'w') as fichier:
                fichier.write(s)
        return s
            
    
    
    def FreeCADExport(self,python_path,fcstd_filepath,path_lib_freecad='',export_types=['fcstd']):
        """
        Export model to .fcstd FreeCAD standard
        
        :param python_path: path of python binded to freecad
             - on windows: 'something like C:\Program Files\FreeCAD X.XX\bin\python'
             - on linux: python (in general)
        :param filepath: path of fcstd file (without extension)
        :param path_lib_freecad: FreeCAD.so lib path (/usr/lib/freecad/lib in general)

        """
        fcstd_filepath=os.path.abspath(fcstd_filepath)
        s=self.FreeCADScript(fcstd_filepath,path_lib_freecad,'',export_types)
#        print(s)
        with tempfile.NamedTemporaryFile(suffix=".py",delete=False) as f:
#        with open(f,'w') as fw:
            f.write(bytes(s,'utf8'))
#            print(f.name)
#            import time
#            time.sleep(20)


#            f.read()
#            print(python_path,arg)
        arg=f.name
        output=subprocess.call([python_path,arg])
#        print(f.name)
        f.close()
#        p = subprocess.Popen('{} {}'.format(python_path,arg), shell=True, stdin=subprocess.PIPE, 
#                  stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
#        output = p.stdout.read()
#        output += p.stderr.read()
        os.remove(f.name)
        return output
        

    
    
    def BabylonScript(self):

        env = Environment(loader=PackageLoader('volmdlr', 'templates'),
                          autoescape=select_autoescape(['html', 'xml']))
        
        template = env.get_template('babylon.html')
        
        center,max_length=self.ModelCaracteristicLengths()
        
        primitives_strings=[]
        for primitive in self.primitives:
            try:
                primitives_strings.append(primitive.Babylon())
            except AttributeError:
                pass
        return template.render(name=self.name,center=tuple(center),length=2*max_length,
                               primitives_strings=primitives_strings)
    
    def BabylonShow(self,page='vm_babylonjs'):
        page+='.html'
#        print(self.BabylonScript())
        with open(page,'w') as file:
            file.write(self.BabylonScript())
        
        webbrowser.open('file://' + os.path.realpath(page))
        
    def ModelCaracteristicLengths(self):
        min_vect=npy.array(self.primitives[0].position)
        max_vect=npy.array(self.primitives[0].position)
        center=npy.array(self.primitives[0].position)
        n=1
        for primitive in self.primitives[1:]:
            try:
                for i,(xmin,xmax,xi) in enumerate(zip(min_vect,max_vect,primitive.position)):
    #                print(i,xmin,xmax,xi)
                    if xi<xmin:
                        min_vect[i]=xi
    #                    print('min',min_vect)
                    if xi>xmax:
                        max_vect[i]=xi
    #                    print('max',max_vect)
    #            print(linkage,linkage.position)
                center+=primitive.position
                n+=1
            except AttributeError:
                pass
                
        center=center/n
#        print(min_vect,max_vect)
        max_length=norm(min_vect-max_vect)
#        print(center,max_length)
        return center,max_length




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 28 14:08:23 2017

@author: steven
"""
import numpy as npy
from scipy.linalg import norm
import volmdlr.geometry as geometry
import math

import matplotlib.pyplot as plt
#import matplotlib.patches as patches
#from matplotlib.collections import PatchCollection

#from volmdlr.primitives2D import Line2D,Arc2D,Point2D
import volmdlr
import volmdlr.geometry as geometry
        
class Cylinder(volmdlr.Primitive3D):
    def __init__(self,position,axis,radius,width,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.position=position
        self.axis=npy.array(axis)/norm(axis)
        self.radius=radius
        self.width=width
        
    def Volume(self):
        return math.pi*self.radius**2*self.width
        
    def FreeCADExport(self,ip):
        if self.radius>0:
            name='primitive'+str(ip)
            e=str(1000*self.width)
            r=str(1000*self.radius)
            position=1000*(self.position-self.axis*self.width/2)
            x,y,z=position
            x=str(x)
            y=str(y)
            z=str(z)
    
            ax,ay,az=self.axis
            ax=str(ax)
            ay=str(ay)
            az=str(az)
            return name+'=Part.makeCylinder('+r+','+e+',fc.Vector('+x+','+y+','+z+'),fc.Vector('+ax+','+ay+','+az+'),360)\n'
        else:
            return ''
    
    def Babylon(self):
        ya,xa,za=self.axis
        theta=math.acos(za/self.width)
        phi=math.atan(ya/xa)
        x,z,y=self.position
        s='var cylinder = BABYLON.Mesh.CreateCylinder("{}", {}, {}, {}, 30, 1, scene,false, BABYLON.Mesh.DEFAULTSIDE);'.format(self.name,self.width,2*self.radius,2*self.radius)
        s+='cylinder.position = new BABYLON.Vector3({},{},{});\n;'.format(x,y,z)
        s+='cylinder.rotation.x={}\n;'.format(-theta*math.sin(phi))
        s+='cylinder.rotation.y={}\n;'.format(theta*math.cos(phi))
        s+='cylinder.rotation.z={}\n;'.format(phi)
        return s
    
class HollowCylinder(volmdlr.Primitive3D):
    def __init__(self,position,axis,inner_radius,outer_radius,width,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.position=position
        self.axis=npy.array(axis)/norm(axis)
        self.inner_radius=inner_radius
        self.outer_radius=outer_radius
        self.width=width
        
    def Volume(self):
        return math.pi*(self.outer_radius**2-self.inner_radius**2)*self.width


    def FreeCADExport(self,ip):
        if self.outer_radius>0.:
            name='primitive'+str(ip)
            re=str(1000*self.outer_radius)
            ri=str(1000*self.inner_radius)        
            position=self.position-self.axis*self.width/2
            x,y,z=1000*position
            ax,ay,az=self.axis
            x=str(x)
            y=str(y)
            z=str(z)
            ax=str(ax)
            ay=str(ay)
            az=str(az)
    #        return 'Part.makeCylinder('+r+','+e+',fc.Vector('+x+','+y+','+z+'),fc.Vector('+ox+','+oy+','+oz+'),360)'
    
            s='C2= Part.makeCircle('+re+',fc.Vector('+x+','+y+','+z+'),fc.Vector('+ax+','+ay+','+az+'))\n'
            s+='W2=Part.Wire(C2.Edges)\n'
            s+='F2=Part.Face(W2)\n'
            
            if self.inner_radius!=0.:
                s+='C1= Part.makeCircle('+ri+',fc.Vector('+x+','+y+','+z+'),fc.Vector('+ax+','+ay+','+az+'))\n'        
                s+='W1=Part.Wire(C1.Edges)\n'
                s+='F1=Part.Face(W1)\n'        
                s+='F2=F2.cut(F1)\n'        
    
            vx,vy,vz=self.axis*self.width*1000
            vx=str(vx)
            vy=str(vy)
            vz=str(vz)
            
            s+=name+'=F2.extrude(fc.Vector('+vx+','+vy+','+vz+'))\n'
            return s
        
        else:
            return ''
    
    def Babylon(self):
        ya,xa,za=self.axis# to counter y definition in babylon
        theta=math.acos(za/self.width)
        phi=math.atan(ya/xa)
        x,z,y=self.position
        s='var cylinder = BABYLON.Mesh.CreateCylinder("{}", {}, {}, {}, 30, 1, scene,false, BABYLON.Mesh.DEFAULTSIDE);'.format(self.name,self.width,2*self.outer_radius,2*self.outer_radius)
        s+='cylinder.position = new BABYLON.Vector3({},{},{});\n;'.format(x,y,z)
        s+='cylinder.rotation.x={}\n;'.format(-theta*math.sin(phi))
        s+='cylinder.rotation.y={}\n;'.format(theta*math.cos(phi))
        s+='cylinder.rotation.z={}\n;'.format(phi)
        return s
    
class ExtrudedProfile(volmdlr.Primitive3D):
    """

    """
    def __init__(self,plane_origin,x,y,contours2D,extrusion_vector,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.contours2D=contours2D
        self.extrusion_vector=extrusion_vector
        self.contours3D=[]
        self.x=x
        self.y=y
        for contour in contours2D:
#            print(contour)
            self.contours3D.append(contour.To3D(plane_origin,x,y))
        
    def MPLPlot(self,ax):
        for contour in self.contours3D:
            for primitive in contour:
                primitive.MPLPlot(ax)
        
    def FreeCADExport(self,ip):
        name='primitive'+str(ip)
        s='W=[]\n'
        for ic,contour in enumerate(self.contours3D): 
            s+='E=[]\n'
            for ip,primitive in enumerate(contour):
                s+=primitive.FreeCADExport('L{}_{}'.format(ic,ip))
                s+='E.append(Part.Edge(L{}_{}))\n'.format(ic,ip)
#            s+='S = Part.Shape(L)\n' 
            s+='W.append(Part.Wire(E[:]))\n'
        s+='F=Part.Face(W)\n'
        e1,e2,e3=self.extrusion_vector
        e1=e1*1000
        e2=e2*1000
        e3=e3*1000
        
        s+='{}=F.extrude(fc.Vector({},{},{}))\n'.format(name,e1,e2,e3)
        return s
    
    def Area(self):
        areas=[c.Area() for c in self.contours2D]
        sic=list(npy.argsort(areas))[::-1]# sorted indices of contours
        area=areas[sic[0]]
        
        for i in sic[1:]:
            area-=self.contours2D[i].Area()
        return area
    
    def Volume(self):
#        e=extrusion_vector/norm(extrusion_vector)
        z=npy.cross(self.x.vector,self.y.vector)
        z/=norm(z)
        coeff=npy.dot(self.extrusion_vector,z)
        
        return self.Area()*coeff
        
        

class Sphere(volmdlr.Primitive3D):
    def __init__(self,center,radius,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.center=center
        self.radius=radius
        self.position=center
    
    def Volume(self):
        return 4/3*math.pi*self.radius**3
    
    def FreeCADExport(self,ip,ndigits=3):
        name='primitive'+str(ip)
        r=1000*self.radius
        x,y,z=npy.round(1000*self.center.vector,ndigits)
        return '{}=Part.makeSphere({},fc.Vector({},{},{}))\n'.format(name,r,x,y,z)


class RevolvedProfile(volmdlr.Primitive3D):
    """
    
    """
    def __init__(self,plane_origin,x,y,contours2D,axis_point,axis,angle=2*math.pi,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.contours2D=contours2D
        self.axis_point=axis_point
        self.axis=axis
        self.angle=angle
        self.plane_origin=plane_origin
        self.x=x
        self.y=y
        
        self.contours3D=[]
        for contour in contours2D:
            self.contours3D.append(contour.To3D(plane_origin,x,y))
        
    def MPLPlot(self,ax):
        for contour in self.contours3D:
            for primitive in contour:
                primitive.MPLPlot(ax)
        
    def FreeCADExport(self,ip,ndigits=3):
        name='primitive'+str(ip)
        s='W=[]\n'
        for ic,contour in enumerate(self.contours3D): 
            s+='L=[]\n'
            for ip,primitive in enumerate(contour):
                s+=primitive.FreeCADExport('L{}_{}'.format(ic,ip),8)
                s+='L.append(L{}_{})\n'.format(ic,ip)
            s+='S = Part.Shape(L)\n' 
            s+='W.append(Part.Wire(S.Edges))\n'
        s+='F=Part.Face(W)\n'
        a1,a2,a3=self.axis.vector
        ap1,ap2,ap3=self.axis_point.vector
        ap1=round(ap1*1000,ndigits)
        ap2=round(ap2*1000,ndigits)
        ap3=round(ap3*1000,ndigits)
        angle=self.angle/math.pi*180
        s+='{}=F.revolve(fc.Vector({},{},{}),fc.Vector({},{},{}),{})\n'.format(name,ap1,ap2,ap3,a1,a2,a3,angle)

#            myObject.Shape = Sweep
        return s
    
    def Volume(self):
        areas=[c.Area() for c in self.contours2D]
        # Maximum area is main surface, others cut into it
        sic=list(npy.argsort(areas))[::-1]# sorted indices of contours
        p1=self.axis_point.PlaneProjection(self.plane_origin,self.x,self.y)
        if self.axis_point.PointDistance(p1)!=0:
            raise NotImplemented
        p1_2D=p1.To2D(self.axis_point,self.x,self.y)
        p2_3D=self.axis_point+volmdlr.Point3D(self.axis.vector)
        p2=p2_3D.PlaneProjection(self.plane_origin,self.x,self.y)
        if p2_3D.PointDistance(p2)!=0:
            raise NotImplemented
        p2_2D=p2_3D.To2D(self.plane_origin,self.x,self.y)
        axis_2D=volmdlr.Line2D(p1_2D,p2_2D)
        
        com=self.contours2D[sic[0]].CenterOfMass()
        rg=axis_2D.PointDistance(com)
        volume=areas[sic[0]]*rg
        
        for i in sic[1:]:
            com=self.contours2D[i].CenterOfMass()
            rg=axis_2D.PointDistance(com)
            volume-=areas[i]*rg
            
        return self.angle*volume
                
        
        
class HelicalExtrudedProfile(volmdlr.Primitive3D):
    """

    """
    def __init__(self,plane_origin,x,y,axis_point,axis,pitch,outer_contour2D,inner_contours2D=[],name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.inner_contours2D=inner_contours2D
        self.outer_contour2D=outer_contour2D
        self.axis_point=axis_point
        self.axis=axis
        self.pitch=pitch
       
        self.inner_contours3D=[c.To3D(plane_origin,x,y) for c in inner_contours2D]
        self.outer_contour3D=outer_contour2D.To3D(plane_origin,x,y)
            
            
    def FreeCADExport(self,ip,ndigits=3):
        name='primitive{}'.format(ip)
        s="E=[]\n"   
        for ip,primitive in enumerate(self.outer_contour3D):
            s+=primitive.FreeCADExport('L_{}'.format(ip))
            s+='E.append(Part.Edge(L_{}))\n'.format(ip)
#        s+='S = Part.Shape(L)\n' 
        s+='W=Part.Wire(E[:])\n'

        a1,a2,a3=self.axis
        ap1,ap2,ap3=self.axis_point
        ap1=round(ap1*1000,ndigits)
        ap2=round(ap2*1000,ndigits)
        ap3=round(ap3*1000,ndigits)
        
        width=norm(self.axis)*1000
        direction=bool(self.pitch < 0)
        pitch=round(abs(self.pitch)*1000,ndigits)
#        print('pitch',pitch)
        s+="helix = Part.makeHelix({}, {}, 50., 0, {})\n".format(pitch,width,direction)
        s+="helix.translate(fc.Vector({},{},{}))\n".format(ap1,ap2,ap3)

        s+='{}=helix.makePipeShell([W],True,True)\n'.format(name)
        for ic,countour in enumerate(self.inner_contours3D):
            s+="Ei=[]\n"   
            s+="helix2 = Part.makeHelix({}, {}, 50., 0, {})\n".format(pitch,1.01*width+pitch,direction)
            s+="helix2.translate(fc.Vector({},{},{}))\n".format(ap1,ap2,ap3-pitch)
            for ip,primitive in enumerate(countour):
                s+=primitive.FreeCADExport('L_{}_{}'.format(ic,ip))
                s+='Ei.append(Part.Edge(L_{}_{}))\n'.format(ic,ip)
            s+='Wi=Part.Wire(Ei[:])\n'
            s+="{}={}.cut(helix2.makePipeShell([Wi],True,True))\n".format(name,name)

        return s
    
class Cut(volmdlr.Primitive3D):
    """

    """
    def __init__(self,primitive,cut_primitive,name=''):
        volmdlr.Primitive3D.__init__(self,name)
        self.primitive=primitive
        self.cut_primitive=cut_primitive
        
    def Volume(self):
        return self.primitive.Volume()-self.cut_primitive.Volume()

        
    def FreeCADExport(self,ip):
        name='primitive{}'.format(ip)
        
        s=self.primitive.FreeCADExport('{}_0'.format(ip))
        s+=self.cut_primitive.FreeCADExport('{}_1'.format(ip))
        
        s+="{}={}_0.cut({}_1)\n".format(name,name,name)

        return s
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar  3 09:56:29 2017

@author: steven
"""

import numpy as npy
import volmdlr as vm
import volmdlr.primitives3D as primitives3D
import volmdlr.primitives2D as primitives2D

p1=vm.Point2D((0,0))
p2=vm.Point2D((0.1,0.))
p3=vm.Point2D((0.1,0.2))
p4=vm.Point2D((0,0.1))

p5=vm.Point2D((0.1,0.3))

l1=primitives2D.RoundedLines2D([p1,p2,p3,p4],{0:0.01,2:0.01})
#l2=vm.Circle2D(p5,0.01)
c1=vm.Contour2D([l1])

po=vm.Point3D((0,0,0))
xp=vm.Vector3D((1,0,0))
yp=vm.Vector3D((0,1,0))
c1.MPLPlot()

profile=primitives3D.ExtrudedProfile(po,xp,yp,[c1],(0,0,0.2))

model=vm.VolumeModel([profile])

#profile.MPLPlot((0,0,0),(1,0,0),(0,1,0))

#model.MPLPlot()

model.FreeCADExport('python','extrusion','/usr/lib/freecad/lib/')


# -*- coding: utf-8 -*-
"""
Setup install script for volmdlr

"""

from setuptools import setup
#from distutils.core import setup
from Cython.Build import cythonize


def readme():
    with open('README.rst') as f:
        return f.read()
    
def version_scheme(version):
    try:
        return '.'.join([str(i) for i in version.tag._key[1]])
    except AttributeError:
        return version.tag
    
#def local_scheme(version):
#    return ''

setup(name='volmdlr',
#      use_scm_version={'version_scheme':version_scheme,'local_scheme':local_scheme},
      use_scm_version = True,
      setup_requires=['setuptools_scm'],
      description=' A volume modeler computation-oriented. Include rendering bindings. ',
      long_description=readme(),
      keywords='volume, modeler',
      url='https://github.com/masfaraud/volmdlr',
      author='Steven Masfaraud',
      author_email='steven@masfaraud.fr',
      license='Creative Commons Attribution-Share Alike license',
      packages=['volmdlr'],#,'volmdlr.primitives2D','volmdlr.primitives3D','volmdlr.geometry'],
      package_dir={},
      install_requires=['numpy','matplotlib','Cython'],
      classifiers=['Topic :: Scientific/Engineering','Development Status :: 3 - Alpha'],
      ext_modules = cythonize("volmdlr/vmcy.pyx"))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  1 14:10:59 2017

@author: steven
"""
import numpy as npy
import math

from scipy.linalg import norm


import volmdlr



class RoundedLines2D(volmdlr.CompositePrimitive2D):
    def __init__(self,points,radius,closed=False,name=''):        
        self.points=points
        self.radius=radius
        self.closed=closed
        # Construncting Arcs and lines of profile
        if closed:
            p1s=[points[-1]]+points[:-1]
            pis=points
            p2s=points[1:]+[points[0]]
        else:
            p1s=[points[-1]]+points[:-2]
            pis=points[:-1]
            p2s=points[1:]
    
        points_l=points[:]
#        primitives=[]
        arcs=[]
        for i in range(len(points)):
            if i in radius:
                r=self.radius[i]
                pt1=p1s[i].vector
                pti=pis[i].vector
                pt2=p2s[i].vector

                dist1=norm(pt1-pti)
                dist2=norm(pt2-pti)
                dist3=norm(pt1-pt2)
                alpha=math.acos(-(dist3**2-dist1**2-dist2**2)/(2*dist1*dist2))/2
                dist=r/math.tan(alpha)

                vec1=(pt1-pti)/dist1
                vec2=(pt2-pti)/dist2

                indice=-vec1[1]*vec2[0]+vec1[0]*vec2[1]

                indice=indice/abs(indice)

                p3=pti+vec1*dist
                p4=pti+vec2*dist

                ptcx=p3[0]-indice*vec1[1]*r
                ptcy=p3[1]+indice*vec1[0]*r
                pc=npy.array((ptcx,ptcy))
                
                d=p3-2*pc+p4
                d=d/norm(d)
                pm=pc+r*d

#                p3c=(p3-pc)
#                p4c=(p4-pc)
#                theta1=npy.arctan2(p3c[1],p3c[0])
#                theta2=npy.arctan2(p4c[1],p4c[0])
#                theta1,theta2=sorted([theta1,theta2])
                points_l[i]=(volmdlr.Point2D(p3),volmdlr.Point2D(p4))                           
                arcs.append(volmdlr.Arc2D(volmdlr.Point2D(p3),volmdlr.Point2D(pm),volmdlr.Point2D(p4)))


        lines=[]
#        if not closed:
#            del points_l[-1]
        try:
            last_point=points_l[0][1]
#            print(points_l)
        except TypeError:
            last_point=points_l[0]
        for p in points_l[1:]+[points_l[0]]:
            if type(p)==tuple:
                lines.append(volmdlr.Line2D(last_point,p[0]))
                last_point=p[1]
            else:
                lines.append(volmdlr.Line2D(last_point,p))
                last_point=p
        if not closed:
            del lines[-1]
            
        primitives=lines[:]
        nlines=len(lines)
        narcs=len(arcs)
        for ii,i in enumerate(sorted(radius.keys(),reverse=True)):
            if i>nlines:
                primitives.append(arcs[narcs-ii-1])
            else:
                primitives.insert(i,arcs[narcs-ii-1])
        
        volmdlr.CompositePrimitive2D.__init__(self,primitives,name)        
        
        
    def Rotation(self,center,angle,copy=True):
        if copy:
            return RoundedLines2D([p.Rotation(center,angle,copy=True) for p in self.points],self.radius,self.closed,self.name)
        else:
            self.__init__([p.Rotation(center,angle,copy=True) for p in self.points],self.radius,self.closed,self.name)
            
    def Translation(self,offset,copy=True):
        if copy:
            return RoundedLines2D([p.Translation(offset,copy=True) for p in self.points],self.radius,self.closed,self.name)
        else:
            self.__init__([p.Translation(offset,copy=True) for p in self.points],self.radius,self.closed,self.name)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 28 15:33:01 2017

@author: steven
"""
import numpy as npy
import volmdlr as vm
import volmdlr.primitives3D as primitives3D

cylinder1=primitives3D.Cylinder((0.,0.,0.),(1.,0.,0.),0.03,0.02,'cylinder1')
cylinder2=primitives3D.HollowCylinder((0,0.1,0.),(1.,0.,0.),0.02,0.06,0.03,'cylinder2')
#profile=primitives3D.ExtrudedProfile((0,0,0),(1,0,0),(0,1,0),[(0,0),(0.1,0.),(0.15,0.4),(0.,0.3)],{0:0.05,2:0.01},(0,0,0.2))

model=vm.VolumeModel([cylinder1,cylinder2])

#profile.MPLPlot((0,0,0),(1,0,0),(0,1,0))

model.FreeCADExport('python','cylinders','/usr/lib/freecad/lib/')

#print(model.BabylonScript())
#model.BabylonShow()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  1 11:14:32 2017

@author: steven
"""
from numpy import dot,cross,array,zeros,random
import volmdlr as vm
import math
from scipy.linalg import norm


def PointProjectionPlane(point,plane_origin,plane_normal):
    """
    Plane is defined by (plane_point,plane_normal)
    :returns : coordinates in global coordinate system
    """
    return vm.Point2D(point-dot(point-plane_origin,plane_normal)*plane_normal)

def PointLocalProjectionPlane(point,plane_origin,x1,x2):
    """
    Plane is defined by (plane_point,x1,x2)
    :returns : coordinates in local coordinate system
    """
    x1=x1.vector
    x2=x2.vector
    point=point.vector
    plane_origin=plane_origin.vector
    plane_normal=cross(x1,x2)
    xp=point-dot(point-plane_origin,plane_normal)*plane_normal-plane_origin# projeted point
    return vm.Point2D((dot(xp,x1),dot(xp,x2)))

def Euler2TransferMatrix(psi,theta,phi):
    """
    Give Transition Matrix from euler angles
    Angles in radians
    """

    cpsi=math.cos(psi)
    spsi=math.sin(psi)
    ctheta=math.cos(theta)
    stheta=math.sin(theta)
    cphi=math.cos(phi)
    sphi=math.sin(phi)
    P=array([[cphi*cpsi-sphi*ctheta*spsi,-spsi*cphi-cpsi*ctheta*sphi,stheta*sphi],[cpsi*sphi+spsi*ctheta*cphi,-sphi*spsi+cphi*ctheta*cpsi,-stheta*cphi],[spsi*stheta,cpsi*stheta,ctheta]])
    return P
    

def TransferMatrix2Euler(R):
    if ((R[2,2]!=1)&(R[2,2]!=-1)):
        theta=math.acos(R[2,2])
        psi=math.atan2(R[2,0]/math.sin(theta),R[2,1]/math.sin(theta))
        phi=math.atan2(R[0,2]/math.sin(theta),-R[1,2]/math.sin(theta))
    else:
        phi=0
        if R[2,2]==1:
            theta=0
            psi=math.atan2(R[1,0],R[0,0])
        else:
            theta=math.pi
            psi=-math.atan2(R[1,0],R[0,0])
    return(array([psi,theta,phi]))

    
def Direction2Euler(u,v=random.random(3)):
#    u=npy.array([ux,uy,uz])
    u=u/norm(u)
    R=zeros((3,3))
    R[:,0]=u
    v=v-dot(u,v)*u
    v=v/norm(v)
    w=cross(u,v)
    R[:,1]=v
    R[:,2]=w
    euler=TransferMatrix2Euler(R)
    return euler


def Huygens2D(I1,area,point1,point2):
    """
    area acts the same way as the mass in 3D
    """
    a,b=point1.vector-point2.vector
    I2=I1+area*array([[b**2,-a*b],[-a*b,a**2]])
    return I2

